import pygame

from pygame.locals import *

from common.Utility import *

from PixelPerfect import _pixelPerfectCollisionDetection


class GameObject:
    """
        Abstract sprite class.
    """

    def __init__(self, myX, myY, myFont):
        """
            Create a new sprite with given x,y position and font.
        """
        self.vx = 0
        self.vy = 0
        self.x = myX
        self.y = myY
        self.camX = 0
        self.camY = 0
        self.image = None
        self.font = myFont
        self.jumpSpeed = 0
        self.canJump = False
        self.isJumping = False
        self.jumpTimer = 500
        self.colliders = []

    def calculateCameraCoords(self, rect):
	"""
	    Calculate the position of this sprite in screen coordinates.
	"""
	self.camX = self.getBounds().x - rect.x
	self.camY = self.getBounds().y - rect.y
	self.colliders = []

    def handleInputs(self, pressLeft, pressRight, pressSpace):
        """
            Respond to pressing left, pressing right, and pressing space.
        """
        pass

    def draw(self, screen):
        """
            Draw this sprite to the screen.
        """
        screen.blit(self.image, (self.camX, self.camY))
        #pygame.draw.rect(screen, (0, 0, 0), pygame.Rect(self.camX, self.camY, self.image.get_width(), self.image.get_height()), 1)

    def getBounds(self):
        """
            Return a rectangular bounding box for the sprite.
        """
        return pygame.Rect(self.x, self.y, self.image.get_width(), self.image.get_height())

    def update(self, dt, others):
        """
            Update the internal logic of this sprite.
            Handle things like collision detection and physics
            dt = time step
            others = list of all sprites in the game world
        """
        pass

    def isSolid(self):
        """
            Return True if this object cannot be passed through.
        """
        return True

    def collide(self, collider):
        """
            If collider has not been collided against already this round,
            handle the collision.
        """
        if self.colliders.count(collider) == 0:
            self.colliders.append(collider)
            self.handleCollision(collider)
            
    def handleCollision(self, collider):
        """
            Respond to the collision of a GameObject and this GameObject.
        """
        pass

    def isDeadly(self):
        """
            Return True if this GameObject can kill Mr. Frog by touching him.
        """
        return False

    def isGoal(self):
        """
            Return True if this GameObject will cause Mr. Frog to win by
            touching it.
        """
        return False

    def isFrog(self):
        """
            Return True if this GameObject is Mr. Frog.
        """
        return False

    def isSnake(self):
        """
            Return True if this GameObject is a snake.
        """
        return False


class Frog(GameObject):
    """
        Mr. Frog sprite.
        This is the sprite the keyboard player controls to play the game with.
        Can jump, turn right and left.
    """

    def __init__(self, x, y, myFont, myMaster):
        """
            Create the Mr. Frog sprite object.
        """
        GameObject.__init__(self, x, y, myFont)
        self.vy = myMaster.getGravity()
        self.animations = getImages("frog")
        self.image = self.animations[0]
        self.master = myMaster
        self.animationCounter = 0
        self.facing = "RIGHT"
        self.onGround = False
        self.standCounter = 100
        self.countStanding = False
        self.holdingJump = False
        self.holdingJumpCounter = 0
        self.startingJumpSpeed = 0
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isFrog(self):
        """
            This is Mr. Frog, return True.
        """
        return True

    def switchAnimation(self, mode):
        """
            Depending on the string 'mode', switch the image of this frog.
            Adjust the position of Mr. Frog to fit with the image used.
        """
        trueVY = self.master.getGravity() + self.jumpSpeed
        oldRect = self.getBounds()
        if mode == "standRight":
            self.image = self.animations[0]
        elif mode == "prepareRight":
            self.image = self.animations[1]
        elif mode == "jumpRight":
            self.image = self.animations[2]
        elif mode == "fallRight":
            self.image = self.animations[3]
        elif mode == "standLeft":
            self.image = self.animations[4]
        elif mode == "prepareLeft":
            self.image = self.animations[5]
        elif mode == "jumpLeft":
            self.image = self.animations[6]
        elif mode == "fallLeft":
            self.image = self.animations[7]

        #when you change images, it is possible that you have grown or shrunk
        #make sure that if you grow, you adjust the position
        #otherwise you could get stuck in blocks
        newRect = self.getBounds()
        if newRect.width > oldRect.width:
            if self.vx > 0:
                #adjust to the left
                newRect.x -= (newRect.width - oldRect.width)
            else:
                #adjust to the right
                newRect.x += (newRect.width - oldRect.width)
        if newRect.height > oldRect.height:
            newRect.y -= (newRect.height - oldRect.height)
        self.x = newRect.x
        self.y = newRect.y
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def handleCollision(self, collider):
        """
            If collider is deadly, end the game for player 2 victory.
            If the collider is a goal, end the game for player 1 victory.
        """
        if collider.isDeadly():
            self.master.handlePlayerDeath()
        elif collider.isGoal():
            self.master.handlePlayerVictory()

    def handleInputs(self, pressLeft, pressRight, pressSpace):
        """
            If press left, face to the left.
            If press right, face to the right.
            If press spacebar for the first time, charge jump.
            When spacebar is released, jump.
            Only jump if on the ground.
        """
        if pressLeft:
            self.facing = "LEFT"
        elif pressRight:
            self.facing = "RIGHT"

        if pressSpace and self.canJump:
            self.holdingJump = True

        if (not pressSpace) and self.canJump and self.holdingJump:
            self.holdingJump = False
            if self.facing == "RIGHT":
                self.switchAnimation("prepareRight")
                self.vx = min(int(self.holdingJumpCounter / 100), 6)
                self.vx += 3
            else:
                self.switchAnimation("prepareLeft")
                self.vx = min(int(self.holdingJumpCounter / 100), 6)
                self.vx *= -1
                self.vx -= 3

            self.canJump = False
            self.startingJumpSpeed = (abs(self.vx) + 5) * -1
            self.jumpSpeed = self.startingJumpSpeed
            self.isJumping = True
            self.jumpTimer = 750
            self.holdingJumpCounter = 0
            self.master.master.playSoundEffect("jump")


    def update(self, dt, others):
        """
            Update Mr. Frog's position based on velocity.
            Handle physics and collision with other sprites.
            Needs to be refactored soon.
        """
        self.onGround = False
        collide = False
        newBounds = self.getBounds()
        vDist = self.master.getGravity() + self.jumpSpeed
        newBounds.y += vDist
        #handle movement in y direction
        for guy in others:
            if guy != self:
                if guy.isDeadly():
                    #only check pixel perfect against snakes and spikes
                    if newBounds.colliderect(guy.getBounds()):
                        if _pixelPerfectCollisionDetection(self, guy):
                            #if collide with deadly, return game is over
                            self.collide(guy)
                            guy.collide(self)
                            collide = True
                            return
                elif guy.isGoal():
                    #if goal, return game is over
                    if newBounds.colliderect(guy.getBounds()):
                        self.collide(guy)
                        guy.collide(self)
                        collide = True
                        return
                else:
                    if newBounds.colliderect(guy.getBounds()):
                        self.collide(guy)
                        guy.collide(self)
                        if guy.isSolid() and self.isSolid():
                            collide = True
                            if self.master.getGravity() + self.jumpSpeed > 0:
                                #collided something while falling.
                                #we have touched the ground, and can therefore jump again
                                self.canJump = True
                                if self.onGround == False:
                                    #switch image back to standing
                                    #on landing, we have no more x velocity
                                    self.vx = 0
                                    self.onGround = True
                                self.jumpSpeed = 0
                                self.isJumping = False
                                self.jumpTimer = 750
                                if self.facing == "RIGHT":
                                    self.switchAnimation("standRight")
                                else:
                                    self.switchAnimation("standLeft")
                                self.y = guy.getBounds().y - self.getBounds().height
                            else:
                                #change this to truly detect if we hit up or down
                                if guy.vy >= 0:
                                    self.y = guy.getBounds().y + guy.getBounds().height
                                else:
                                    self.y = guy.getBounds().y + guy.vy - self.getBounds().height
                                #we were moving up and hit a ceiling
                                #stop jumping, now just fall
                                self.jumpSpeed = 0
                                self.isJumping = False
                                self.jumpTimer = 750

                                break

        if not collide:
            #if didn't hit anything, just move as far as we can
            self.y = newBounds.y

        collide = False
        newBounds = self.getBounds()
        newBounds.x += self.vx


        #handle movement in x direction
        for guy in others:
            if guy != self:
                if guy.isDeadly():
                    if newBounds.colliderect(guy.getBounds()):
                        if _pixelPerfectCollisionDetection(self, guy):
                            #game is over if hit a snake/spike, so return
                            self.collide(guy)
                            guy.collide(self)
                            collide = True
                            return
                else:
                    if newBounds.colliderect(guy.getBounds()):
                        self.collide(guy)
                        guy.collide(self)
                        if guy.isGoal():
                            #game is over if hit goal, so return
                            return
                        elif guy.isSolid() and self.isSolid():
                            collide = True
                            if self.vx > 0:
                                self.x = (guy.getBounds().x) - self.getBounds().width
                                break
                            else:
                                self.x = guy.getBounds().x + guy.getBounds().width
                                break
        #if you didnt hit a wall, go as far as possible
        if not collide:
            self.x = newBounds.x

        #keep track of how long the user is holding down jump button
        if self.holdingJump:
            self.holdingJumpCounter += dt
        #handle jump logic
        if self.isJumping:
            self.jumpTimer -= dt
            self.jumpSpeed = int((self.startingJumpSpeed * (self.jumpTimer / 750.0)))
            if self.jumpTimer <= 0:
                self.jumpTimer = 750
                self.isJumping = False
                self.jumpSpeed = 0
            elif self.jumpTimer <= 650:
                if self.facing == "RIGHT":
                    self.switchAnimation("jumpRight")
                else:
                    self.switchAnimation("jumpLeft")

        self.master.scroll(self.getBounds(), max(abs(self.vx), abs(self.master.getGravity() + self.jumpSpeed)))

class Grass(GameObject):
    """
        A sprite with grass on top and dirt on bottom.
        Normal movement blocking sprite.
    """

    def __init__(self, x, y, myFont):
        """
            Create a Grass sprite.
        """
        GameObject.__init__(self, x, y, myFont)
        self.image = splitFeldman(0, 2, 10)

class Wall(GameObject):
    """
        A sprite made out of dirt.
        Normal movement blocking sprite.
    """

    def __init__(self, x, y, myFont):
        """
            Create a Wall sprite.
        """
        GameObject.__init__(self, x, y, myFont)
        self.image = splitFeldman(0, 5, 10)

                
class Spike1(GameObject):
    """
        Sharp pointy thing that can kill Mr. Frog.
        Points up.
    """

    def __init__(self, x, y, myFont):
        """
            Create Spike sprite.
        """
        GameObject.__init__(self, x, y, myFont)
        #pointy up
        self.image = splitFeldman(1, 12, 4)
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isDeadly(self):
        """
            Yeah, I wouldn't touch it.
            Return True.
        """
        return True

class Spike2(GameObject):
    """
        Sharp pointy thing that can kill Mr. Frog.
        Points left.
    """

    def __init__(self, x, y, myFont):
        GameObject.__init__(self, x, y, myFont)
        #pointy to left
        self.image = splitFeldman(1, 12, 4)
        self.image = pygame.transform.rotate(self.image, 90)
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isDeadly(self):
        """
            Yeah, I wouldn't touch it.
            Return True.
        """
        return True

class Spike3(GameObject):
    """
        Sharp pointy thing that can kill Mr. Frog.
        Points down.
    """

    def __init__(self, x, y, myFont):
        GameObject.__init__(self, x, y, myFont)
        #pointy down
        self.image = splitFeldman(1, 12, 4)
        self.image = pygame.transform.rotate(self.image, 180)
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isDeadly(self):
        """
            Yeah, I wouldn't touch it.
            Return True.
        """
        return True

class Spike4(GameObject):
    """
        Sharp pointy thing that can kill Mr. Frog.
        Points right.
    """

    def __init__(self, x, y, myFont):
        GameObject.__init__(self, x, y, myFont)
        #pointy to right
        self.image = splitFeldman(1, 12, 4)
        self.image = pygame.transform.rotate(self.image, 270)
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isDeadly(self):
        """
            Yeah, I wouldn't touch it.
            Return True.
        """
        return True

class Goal(GameObject):
    """
        The Sacred Platform - goal of Mr. Frog's struggles.
        When it is touched by Mr. Frog, Mr. Frog wins.
    """

    def __init__(self, x, y, myFont):
        """
            Create a goal sprite.
        """
        GameObject.__init__(self, x, y, myFont)
        self.image = splitFeldman(0, 0, 10)

    def isGoal(self):
        """
            Return True.
        """
        return True

class Snake(GameObject):
    """
        This is a Snake.
        It can kill Mr. Frog.
        It is summoned out of thin air by OSIRIS
    """

    def __init__(self, x, y, myFont, m):
        """
            Create a new Snake Sprite.
            Starts moving to the left.
        """
        GameObject.__init__(self, x, y, myFont)
        self.master = m
        self.leftImages = getImages("snakeLeft")
        self.rightImages = getImages("snakeRight")
        self.imageIndex = 0
        self.image = self.leftImages[self.imageIndex]
        self.facing = "LEFT"
        self.animationCounter = 0
        self.animationMax = 200
        self.vy = self.master.getGravity()
        self.vx = -3
        self.hitmask = pygame.surfarray.array_alpha(self.image)

    def isSnake(self):
        """
            Q:  How do you speak Python?
            A:  hissssssssssssssssssssssssss
            Return True.
        """
        return True

    def advanceAnimation(self):
        """
            Advance slithering animation.
            Adjust bounds so that it doesn't accidentally collide by expanding.
        """
        self.imageIndex += 1
        oldRect = self.getBounds()
        if self.imageIndex == len(self.rightImages):
            self.imageIndex = 0
        if self.facing == "LEFT":
            self.image = self.leftImages[self.imageIndex]
        else:
            self.image = self.rightImages[self.imageIndex]
            

        #when you change images, it is possible that you have grown or shrunk
        #make sure that if you grow, you adjust the position
        #otherwise you could get stuck in blocks
        newRect = self.getBounds()
        newRect.x = oldRect.x
        newRect.y = oldRect.y
        if newRect.width > oldRect.width:
            if self.vx > 0:
                #adjust to the left
                newRect.x -= (newRect.width - oldRect.width)
            else:
                #adjust to the right
                newRect.x += (newRect.width - oldRect.width)
        if newRect.height > oldRect.height:
            if self.master.getGravity() > 0:
                #adjust up
                newRect.y -= (newRect.height - oldRect.height)
            else:
                #adjust down
                newRect.y += (newRect.height - oldRect.height)
        self.x = newRect.x 
        self.y = newRect.y
        self.hitmask = pygame.surfarray.array_alpha(self.image)


    def update(self, dt, others):
        """
            Handle internal logic of snake.
            Basically, move in one direction until you hit a wall.
            Then turn around and go the other way.
            I never said snakes were smart, just deadly.
            Needs to be refactored soon.
        """
        #as it slithers, adjust animation
        self.animationCounter += dt
        if self.animationCounter > self.animationMax:
            self.advanceAnimation()
            self.animationCounter = 0
        #move in x direction
        newBounds = self.getBounds()
        newBounds.x += self.vx
        collide = False
        for guy in others:
            if guy != self:
                if guy.isFrog():
                    if newBounds.colliderect(guy.getBounds()):
                        if _pixelPerfectCollisionDetection(self, guy):
                            #if hit frog, game over so return
                            self.collide(guy)
                            guy.collide(self)
                            return
                else:
                    if newBounds.colliderect(guy.getBounds()):
                        self.collide(guy)
                        guy.collide(self)
                        if guy.isSolid() and self.isSolid():
                            collide = True
                            self.turnAround()
                            self.animationCounter = 0
                            return

        if not collide:
            #if no collision, move as far as possible
            self.x = newBounds.x
        #check movement in y direction
        collide = False
        newBounds = self.getBounds()
        newBounds.y += self.master.getGravity()
        for guy in others:
            if guy != self:
                if guy.isFrog():
                    if newBounds.colliderect(guy.getBounds()):
                        if _pixelPerfectCollisionDetection(self, guy):
                            #if hit frog, game over so return
                            self.collide(guy)
                            guy.collide(self)
                            return
                else:
                    if newBounds.colliderect(guy.getBounds()):
                        self.collide(guy)
                        guy.collide(self)
                        if guy.isSolid() and self.isSolid():
                            collide = True
                            if self.master.getGravity() > 0:
                                self.y = guy.getBounds().y - self.getBounds().height
                                break
                            else:
                                self.y = guy.getBounds().y + guy.getBounds().height
                                break
        if not collide:
            #if no collision, move as far as possible in y direction
            self.y = newBounds.y



    def turnAround(self):
        """
            Turn the snake around.
            Change velocity and change iamge / animation sequence.
        """
        if self.facing == "RIGHT":
            self.facing = "LEFT"
        else:
            self.facing = "RIGHT"
        self.advanceAnimation()
        self.vx *= -1
        

    def isDeadly(self):
        """
            Of course its deadly, its a freaking snake.
        """
        return True
        

class HiddenBlock(GameObject):
    """
        Block which sneakily has the same colorscheme as background.
        Very sneaky.
    """

    def __init__(self, x, y, myFont):
        """
            Create HiddenBlock sprite.
        """
        GameObject.__init__(self, x, y, myFont)
        self.image = splitFeldman(0, 5, 6)

    def update(self, dt, others):
        """
            Has no internal logic.
        """
        pass

class Background:
    """
        Repeats one small 32x32 image infinitely to form a background.
        Should be blitted first to create an illusion of depth.
    """

    def __init__(self, width, height, key, x, y):
        """
            Create  a Background image for blitting.
            width = how wide to make the sprite
            height = how tall to make the sprite
            key = which feldman sprite sheet to split
            x, y = index into sprite sheet
        """
        self.image = pygame.Surface((width, height), flags=SRCALPHA)
        repeater = splitFeldman(key, x, y)
        for i in xrange(width / 32):
            for j in xrange(height / 32):
                self.image.blit(repeater, (i * 32, j * 32))

    def draw(self, surface):
        """
            Draw this background image to the screen.
        """
        surface.blit(self.image, (0, 0))

        
        
