import numpy
import pygame
from pygame.locals import *

from pgu import gui

from Avatar import Frog

from Avatar import Wall

from Avatar import Grass

from Avatar import Spike1

from Avatar import Spike2

from Avatar import Spike3

from Avatar import Spike4

from Avatar import Goal

from Avatar import Background

from Avatar import Snake

from Avatar import HiddenBlock

from common.Utility import *

import sys

class PlatformerLevel:
    """
        Contains internal game logic.
        Draws, updates sprite logic, and handles inputs from players.
    """

    def __init__(self, **params):
        """
            Constructor.
            Create relevant data for game.
        """
        self.goingLeft = False
        self.goingRight = False
        self.pressingSpace = False
        self.master = params.get('myMaster', 0)
        self.fname = params.get("filename", 0)
        self.font = pygame.font.SysFont("Times New Roman", 50, bold=True)

        self.originalOuter = pygame.Rect(0, 0, 1000, 400)
        self.originalInner = pygame.Rect(50, 100, 100, 200)

        self.outer = self.originalOuter
        self.inner = self.originalInner

        self.objects = []



        self.PAUSE_COUNTER = 4000
        self.pauseTimer = 0
        self.pausedForDeath = False
        self.pausedForVictory = False
        self.backgroundLayer = Background(1200, 500, 0, 5, 6)
        self.placingSnake = False
        self.canPlaceSnake = False
        #static images to draw
        self.snakeImage = pygame.image.load("data/images/snakeLeft3.png")
        self.noSnakeImage = pygame.image.load("data/images/noSnake.png")
        self.clickToStartImage = pygame.image.load("data/images/clickToStart.png")
        self.gravityUpImage = pygame.image.load("data/images/gravityUp.png")
        self.gravityDownImage = pygame.image.load("data/images/gravityDown.png")

        self.darkness = False
        self.darknessCounter = 0
        self.darknessCounterMax = 3000

        self.powerRefillCounter = 0
        self.powerRefillCounterMax = 2000

        self.gravityLow = 1
        self.gravityNormal = 3
        self.gravityHigh = 5

        self.gravity = self.gravityNormal

        self.alterGravity = False
        self.gravityTimer = 0
        self.gravityTimerMax = 3000

        self.currentTime = 0

        self.frog = None

        self.pausedForBegin = True

        self.frogName = None
        self.gozerName = None
        self.previousWinner = None
        self.transparentCircle = pygame.Surface((125, 125)).convert_alpha()
        self.transparentCircle.fill((0, 0, 0, 0))
        pygame.draw.circle(self.transparentCircle, (255, 255, 255, 128), (63, 63), 50)
        
        self.buildWorld(self.fname)

    def getGravity(self):
        """
            Accessor method for current gravity ammount.
        """
        return self.gravity

    def reset(self):
        """
            Build the world for gameplay.
            What level is built depends on class variable 'self.fname'.
            Pause for the user to start.
        """
        self.gravity = self.gravityNormal
        self.alterGravity = False
        self.darkness = False
        self.darknessCounter = 0
        self.powerRefillCounter = 0
        self.gravityTimer = 0
        self.master.setPower(50)
        self.powerRefillCounter = 0
        self.placingSnake = False
        self.canPlaceSnake = False
        self.outer = self.originalOuter
        self.inner = self.originalInner
        self.objects = []
        self.buildWorld(self.fname)
        self.master.addBeginAndBack()
        self.pausedForBegin = True

    def handlePlayerDeath(self):
        """
            Game ended with frog death.
            Play loser sound effect, pause for death.
        """
        self.pausedForDeath = True
        self.darkness = False
        self.placingSnake = False
        self.previousWinner = self.gozerName
        self.master.playSoundEffect("gozerWins")

    def handlePlayerVictory(self):
        """
            Game ended with frog reaching sacred platform.
            Player winner sound effect, pause for victory.
        """
        self.pausedForVictory = True
        self.placingSnake = False
        self.darkness = False
        self.previousWinner = self.frogName
        self.master.playSoundEffect("frogWins")

    def buildWorld(self, fileToOpen):
        """
            Parse a level file and create a game world based on the contents.
            Tile based level files are big matrices of ASCII characters.
            The row, column of the matrix is the x, y location.
            Each tile in the game is 32 pixels wide and 32 pixels tall.
        """
        f = open(fileToOpen)
        for y, line in enumerate(f.readlines()):
            for x, c in enumerate(line):
                if c == "g":
                    #grass
                    self.objects.append(Grass(32 * x, 32 * y, self.font))
                elif c == "w":
                    #wall
                    self.objects.append(Wall(32 * x, 32 * y, self.font))
                elif c == "s":
                    #spike, facing up
                    self.objects.append(Spike1(32 * x, 32 * y, self.font))
                elif c == "2":
                    #spike, facing left
                    self.objects.append(Spike2(32 * x, 32 * y, self.font))
                elif c == "3":
                    #spike, facing down
                    self.objects.append(Spike3(32 * x, 32 * y, self.font))
                elif c == "4":
                    #spike, facing right
                    self.objects.append(Spike4(32 * x, 32 * y, self.font))
                elif c == "x":
                    #sacred platform
                    self.objects.append(Goal(32 * x, 32 * y, self.font))
                elif c == "b":
                    #hidden block to match style of background
                    self.objects.append(HiddenBlock(32 * x, 32 * y, self.font))
                elif c == "f":
                    #mr frog, should only be one of these per file
                    #adjust screen coordinates to center near frog
                    self.frog = Frog(32 * x, 32 * y, self.font, self)
                    self.inner.centerx = self.frog.getBounds().centerx + 200
                    self.inner.centery = self.frog.getBounds().centery
                    self.outer.centerx = self.frog.getBounds().centerx + 200
                    self.outer.centery = self.frog.getBounds().centery
                    self.objects.append(self.frog)
                    
        f.close()

    def handleSnake(self):
        """
            Osiris is trying to summon a snake.
            Check if the user can, if they can move into place mode.
        """
        if not self.pausedForBegin:
            if self.master.getPower() >= 50 and not self.placingSnake:
                self.master.setPower(self.master.getPower() - 50)
                self.placingSnake = True

    def handleDarkness(self):
        """
            Osiris is trying to summon an eclipse.
            Check if the user can, if true then turn screen black.
        """
        if not self.pausedForBegin:
            if self.master.getPower() >= 50:
                self.master.playSoundEffect("eclipse")
                self.master.setPower(self.master.getPower() - 50)
                self.darkness = True

    def handleGravityUp(self):
        """
            Osiris is trying to make gravity increase.
            Check if possible, if true then turn gravity to high and
            start countdown to reset to normal gravity.
        """
        if not self.pausedForBegin:
            if self.master.getPower() >= 25:
                self.master.playSoundEffect("gravityUp")
                self.master.setPower(self.master.getPower() - 25)
                self.alterGravity = True
                self.gravity = self.gravityHigh

    def handleGravityDown(self):
        """
            Osiris is trying to make gravity decrease.
            Check if possible, if true then turn gravity to low and
            start countdown to reset to normal gravity.
        """
        if not self.pausedForBegin:
            if self.master.getPower() >= 25:
                self.master.playSoundEffect("gravityDown")
                self.master.setPower(self.master.getPower() - 25)
                self.alterGravity = True
                self.gravity = self.gravityLow


    def tick(self, dt):
        """
            Update game logic by a time step.
            dt = length of timestep in milliseconds
        """
        if self.pausedForDeath:
            #player has died, display death animation
            self.pauseTimer += dt
            if self.pauseTimer >= self.PAUSE_COUNTER:
                self.pausedForDeath = False
                self.pauseTimer = 0
                self.reset()
        elif self.pausedForVictory:
            #player has won, display victory screen
            self.pauseTimer += dt
            if self.pauseTimer >= self.PAUSE_COUNTER:
                self.pausedForVictory = False
                self.pauseTimer = 0
                self.reset()
        elif self.pausedForBegin:
            #wait for user to click start button
            pass
        else:
            self.currentTime += dt
            #normal gameplay
            #update logic for each sprite
            for obj in self.objects:
                obj.calculateCameraCoords(self.outer)
                obj.handleInputs(self.goingLeft, self.goingRight, self.pressingSpace)
                obj.update(dt, self.objects)

            #refill mana bar for OSIRIS
            self.powerRefillCounter += dt
            if self.powerRefillCounter >= self.powerRefillCounterMax:
                self.master.setPower(self.master.getPower() + 5)
                self.powerRefillCounter = 0

            #turn screen back to normal after 3 seconds
            if self.darkness:
                self.darknessCounter += dt
                if self.darknessCounter >= self.darknessCounterMax:
                    self.darknessCounter = 0
                    self.darkness = False
                    self.master.playSoundEffect("eclipse")

            #turn gravity back to normal after 3 seconds
            if self.alterGravity:
                self.gravityTimer += dt
                if self.gravityTimer >= self.gravityTimerMax:
                    self.gravityTimer = 0
                    self.gravity = self.gravityNormal
                    self.alterGravity = False

            #if user is trying to place the snake
            #check if the snake is on the right side of screen
            #and the frog is on the ground
            #and make sure it doesnt impact any other sprites when created
            #if all of above are true, let the user place snake if she clicks
            if self.placingSnake:
                pos = pygame.mouse.get_pos()
                snakeRect = pygame.Rect(0, 0, self.snakeImage.get_width(), self.snakeImage.get_height())
                snakeRect.centerx = pos[0]
                snakeRect.centery = pos[1]
                self.canPlaceSnake = False
                if snakeRect.centerx >= 900 and self.frog.onGround:
                    impact = False
                    trueBounds = pygame.Rect(snakeRect.x + self.outer.x, snakeRect.y + self.outer.y, snakeRect.width, snakeRect.height)
                    for dude in self.objects:
                        if trueBounds.colliderect(dude.getBounds()):
                            impact = True
                    if not impact:
                        self.canPlaceSnake = True

            

    def inView(self):
        """
            Return all sprites in view of the camera.
        """
        viewable = []
        for sprite in self.objects:
            if sprite.getBounds().colliderect(self.outer):
                sprite.calculateCameraCoords(self.outer)
                viewable.append(sprite)
        return viewable

    def scroll(self, rect, speed):
        """
            Scroll the camera bounds by a given speed.
            Ensure that 'rect' is visible.
        """
        if not self.inner.contains(rect):
            if rect.x >= self.inner.x + self.inner.width:
                #if self.outerCamX + speed < self.worldWidth:
                self.outer.x += speed
                self.inner.x += speed
            elif rect.x <= self.inner.x:		
		#if self.outerCamX - speed > 0:
                self.outer.x -= speed
                self.inner.x -= speed
            if rect.y >= self.inner.y + self.inner.height:
                self.outer.y += speed
                self.inner.y += speed
            elif rect.y <= self.inner.y:
                self.outer.y -= speed
                self.inner.y -= speed


    def drawGame(self, screen):
        """
            Draw all of the game sprites and special effects to screen.
        """
        #make screen all red for 0.5 seconds if dead
        if self.pausedForDeath and self.pauseTimer >= 500:
            screen.fill((255, 0, 0))
            self.frog.draw(screen)
        elif self.pausedForVictory:
            #draw a halo around frog for victory special effect
            self.backgroundLayer.draw(screen)
            for obj in self.inView():
                if obj.isFrog() == False:
                    obj.draw(screen)
            screen.blit(self.transparentCircle, (self.frog.camX + (self.frog.getBounds().width / 2) - (self.transparentCircle.get_width() / 2), self.frog.camY + (self.frog.getBounds().height / 2) - (self.transparentCircle.get_height() / 2)))
            self.frog.draw(screen)
        else:
            #normal game draw sequence
            #draw background
            self.backgroundLayer.draw(screen)
            #draw all game sprites
            for obj in self.inView():
                obj.draw(screen)
            #draw attempt for snake placement
            if self.placingSnake:
                pos = pygame.mouse.get_pos()
                snakeRect = pygame.Rect(0, 0, self.snakeImage.get_width(), self.snakeImage.get_height())
                snakeRect.centerx = pos[0]
                snakeRect.centery = pos[1]
                if self.canPlaceSnake:
                #if snakeRect.centerx >= 900:
                    screen.blit(self.snakeImage, snakeRect)
                else:
                    screen.blit(self.noSnakeImage, snakeRect)
            #draw gravity change visual cue
            if self.alterGravity:
                if self.gravity == self.gravityHigh:
                    screen.blit(self.gravityUpImage, (115, 15))
                else:
                    screen.blit(self.gravityDownImage, (115, 15))
            #draw eclipse
            if self.darkness:
                screen.fill((0, 0, 0))
            #draw time remaining
            timeImage = self.font.render(str(self.currentTime / 1000.0), False, (255, 255, 255))
            screen.blit(timeImage, (500, 50))
            #draw victory text
            if self.pausedForBegin and self.previousWinner != None:
                winnerImage = self.font.render(self.previousWinner + " wins!", False, (255, 255, 255))
                screen.blit(winnerImage, (200, 200))
            
            

    def handleEvent(self, e):
        """
            Handle user input from mouse and keyboard.
        """
        if e.type == KEYDOWN:
            if e.key == K_LEFT:
                #try to turn left
                self.goingLeft = True
                self.goingRight = False
            elif e.key == K_RIGHT:
                #true to turn right
                self.goingLeft = False
                self.goingRight = True
            elif e.key == K_SPACE:
                #try to charge jump
                self.pressingSpace = True
        elif e.type == KEYUP:
            if e.key == K_LEFT:
                #no longer trying to turn left
                self.goingLeft = False
            elif e.key == K_RIGHT:
                #no longer trying to turn right
                self.goingRight = False
            elif e.key == K_SPACE:
                #release spacebar, try to jump
                self.pressingSpace = False
        elif e.type == MOUSEBUTTONDOWN:
            #osiris clicked mouse
            if self.placingSnake:
                #try to place snake
                if self.canPlaceSnake:
                    #summon a snake at the position desired
                    pos = pygame.mouse.get_pos()
                    snakeRect = pygame.Rect(0, 0, self.snakeImage.get_width(), self.snakeImage.get_height())
                    snakeRect.centerx = pos[0]
                    snakeRect.centery = pos[1]
                    trueBounds = pygame.Rect(snakeRect.x + self.outer.x, snakeRect.y + self.outer.y, snakeRect.width, snakeRect.height)
                    s = Snake(trueBounds.x, trueBounds.y, self.font, self)
                    self.objects.append(s)
                    self.placingSnake = False
                    self.canPlaceSnake = False
                    self.master.playSoundEffect("summon")




class FrogContainer(gui.Container):
    """
        Main GUI for game.
        Handles into/credits/instructions/stage select screens.
    """

    def __init__(self):
        """
            Size of game GUI will be 1200x400.
            Create all widgets and start at the intro screen.
        """
        gui.Container.__init__(self, width=1200, height=400)

        #start screen widgets
        self.playButton = gui.Button("PLAY", width=200, height=50)
        self.playButton.connect(gui.CLICK, self.stageSelect, None)
        self.instructionsButton = gui.Button("INSTRUCTIONS", width=200, height=50)
        self.instructionsButton.connect(gui.CLICK, self.frogInstructions, None)
        self.creditsButton = gui.Button("CREDITS", width=200, height=50)
        self.creditsButton.connect(gui.CLICK, self.creditsScreen, None)
        self.quitButton = gui.Button("QUIT", width=200, height=50)
        self.quitButton.connect(gui.CLICK, self.exitGame, None)
        self.startImage = gui.Image("data/images/intro.png", width=1200, height=300)

        #mr frog instruction page widgets
        self.frog2Gozer = gui.Button("Instructions for OSIRIS", width=100, height=50)
        self.frog2Gozer.connect(gui.CLICK, self.gozerInstructions, None)
        self.frogInstructionP = gui.Image("data/images/frogInstructions.png", width=1200, height=350)
        self.frog2Title = gui.Button("Back to Menu", width=150, height=50)
        self.frog2Title.connect(gui.CLICK, self.startScreen, None)

        #GOZER instruction page widgets
        self.gozer2Frog = gui.Button("Instructions for Mr. Frog", width=100, height=50)
        self.gozer2Frog.connect(gui.CLICK, self.frogInstructions, None)
        self.gozerInstructionP = gui.Image("data/images/gozerInstructions.png", width=1200, height=350)
        #reuse frog2Title

        #credits screen

        creditsString = "Game Design:  Jay Reinebold\n(www.reinebold.com)\n\n"
        creditsString += "Built with Pygame\n(www.pygame.org)\n\n"
        creditsString += "Uses the PGU Library\n (http://code.google.com/p/pgu/)\n\n"
        creditsString += "Uses the PixelPerfect Library\n (http://www.pygame.org/project-PixelPerfect-207-.html)\n\n"
        creditsString += "Artwork:  Ari Feldman's Open Sprite Library (http://www.flyingyogi.com/fun/spritelib.html)\n\n"
        creditsString += "Sound Effects:  www.freesound.org\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=61847\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=85245\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=69546\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=38317\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=38639\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=87043\n"
        creditsString += "http://www.freesound.org/samplesViewSingle.php?id=73595\n\n"
        creditsString += "Music:  http://incompetech.com/m/c/royalty-free\n\n"
        creditsString += "Testing:\n"
        creditsString += "Anthony Bart\nDan Bart\nJohn 'the Road Warrior' Bart\n\n"
        creditsString += "Enjoy the game!"


        self.creditsText = gui.TextArea(value=creditsString, width=600, height=650, focusable=False)
        self.creditsTextArea = gui.ScrollArea(widget=self.creditsText, width=600, height=350)
        
        #reuse back to start


        #stage select widgets
        self.levelImage = gui.Image("data/images/vs.png", width=625, height=200)
        self.playerOneInput = gui.Input("Player 1", width=200, height=50)
        self.playerOneTagline = gui.Label("...likes lily pads. (keyboard)", width=200, height=50)
        self.playerTwoInput = gui.Input("Player 2", width=200, height=50)
        self.playerTwoTagline = gui.Label("...eater of worlds. (mouse)", width=200, height=50)
        self.selectLevelOne = gui.Button("GRASSY PLAIN", width=150, height=50)
        self.selectLevelOne.connect(gui.CLICK, self.startLevel, "data/levels/level1.txt")
        self.selectLevelOne.connect(gui.ENTER, self.hoverButton, "level1")
        self.selectLevelOne.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelTwo = gui.Button("PSYCHO LILY PAD", width=150, height=50)
        self.selectLevelTwo.connect(gui.CLICK, self.startLevel, "data/levels/level2.txt")
        self.selectLevelTwo.connect(gui.ENTER, self.hoverButton, "level2")
        self.selectLevelTwo.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelThree = gui.Button("TUNNEL SNAKES", width=150, height=50)
        self.selectLevelThree.connect(gui.CLICK, self.startLevel, "data/levels/level3.txt")
        self.selectLevelThree.connect(gui.ENTER, self.hoverButton, "level3")
        self.selectLevelThree.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelFour = gui.Button("UP AND DOWN", width=150, height=50)
        self.selectLevelFour.connect(gui.CLICK, self.startLevel, "data/levels/level4.txt")
        self.selectLevelFour.connect(gui.ENTER, self.hoverButton, "level4")
        self.selectLevelFour.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelFive = gui.Button("THE BRIDGE", width=150, height=50)
        self.selectLevelFive.connect(gui.CLICK, self.startLevel, "data/levels/level5.txt")
        self.selectLevelFive.connect(gui.ENTER, self.hoverButton, "level5")
        self.selectLevelFive.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelSix = gui.Button("TEMPLE", width=150, height=50)
        self.selectLevelSix.connect(gui.CLICK, self.startLevel, "data/levels/level6.txt")
        self.selectLevelSix.connect(gui.ENTER, self.hoverButton, "level6")
        self.selectLevelSix.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelSeven = gui.Button("HUH?", width=150, height=50)
        self.selectLevelSeven.connect(gui.CLICK, self.startLevel, "data/levels/level7.txt")
        self.selectLevelSeven.connect(gui.ENTER, self.hoverButton, "level7")
        self.selectLevelSeven.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelEight = gui.Button("CHOICES", width=150, height=50)
        self.selectLevelEight.connect(gui.CLICK, self.startLevel, "data/levels/level8.txt")
        self.selectLevelEight.connect(gui.ENTER, self.hoverButton, "level8")
        self.selectLevelEight.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelNine = gui.Button("DROP", width=150, height=50)
        self.selectLevelNine.connect(gui.CLICK, self.startLevel, "data/levels/level9.txt")
        self.selectLevelNine.connect(gui.ENTER, self.hoverButton, "level9")
        self.selectLevelNine.connect(gui.EXIT, self.leaveButton, None)
        self.selectLevelTen = gui.Button("SO CLOSE", width=150, height=50)
        self.selectLevelTen.connect(gui.CLICK, self.startLevel, "data/levels/level10.txt")
        self.selectLevelTen.connect(gui.ENTER, self.hoverButton, "level10")
        self.selectLevelTen.connect(gui.EXIT, self.leaveButton, None)
        self.stageSelectToStartScreen = gui.Button("Back", width=100, height=25)
        self.stageSelectToStartScreen.connect(gui.CLICK, self.startScreen, None)
        
        #game widgets
        self.level = None
        self.snakeButton = gui.Button(gui.Image("data/images/snakeLeft3.png"), width=200, height=100)
        self.snakeButton.connect(gui.CLICK, self.snakeClicked, None)
        self.darknessButton = gui.Button(gui.Image("data/images/darkness.png"), width=200, height=100)
        self.darknessButton.connect(gui.CLICK, self.darknessClicked, None)
        self.gravityUpButton = gui.Button(gui.Image("data/images/gravityUp.png"), width=200, height=100)
        self.gravityUpButton.connect(gui.CLICK, self.gravityUpClicked, None)
        self.gravityDownButton = gui.Button(gui.Image("data/images/gravityDown.png"), width=200, height=100)
        self.gravityDownButton.connect(gui.CLICK, self.gravityDownClicked, "WHAT")
        self.dietyPowerBar = gui.ProgressBar(50, 0, 100, width=400, height=25)
        self.beginButton = gui.Button("GO!", width = 200, height = 50)
        self.beginButton.connect(gui.CLICK, self.beginStage, None)
        self.backButton = gui.Button("Back to Menu", width= 200, height = 50)
        self.backButton.connect(gui.CLICK, self.backToStageSelect, None)

        #initialize sound
        pygame.mixer.quit()
        pygame.mixer.init()
        self.currentScreen = "NONE"
        self.menuSound = pygame.mixer.Sound(load_sound("menu.ogg"))
        self.frogWinsSound = pygame.mixer.Sound(load_sound("frogWins.ogg"))
        self.eclipseSound = pygame.mixer.Sound(load_sound("eclipse.ogg"))
        self.gozerWinsSound = pygame.mixer.Sound(load_sound("gozerWins.ogg"))
        self.gravityUpSound = pygame.mixer.Sound(load_sound("gravityUp.ogg"))
        self.gravityDownSound = pygame.mixer.Sound(load_sound("gravityDown.ogg"))
        self.jumpSound = pygame.mixer.Sound(load_sound("jump.ogg"))
        self.levelSound = pygame.mixer.Sound(load_sound("level.ogg"))
        self.summonSound = pygame.mixer.Sound(load_sound("summon.ogg"))
        #play menu background music
        self.menuSound.play(loops=-1)
        #start at the start screen
        self.startScreen(None)

    def hoverButton(self, param):
        """
            Show the correct stage preview if the user hovers over a button.
        """
        self.levelImage.value = pygame.image.load("data/images/preview" + param + ".png")

    def leaveButton(self, param):
        """
            If user is not hovering over a button, display vs image.
        """
        self.levelImage.value = pygame.image.load("data/images/vs.png")

    def playSoundEffect(self, name):
        """
            Play a sound effect based on name.
        """
        if name == "summon":
            self.summonSound.play()
        elif name == "jump":
            self.jumpSound.play()
        elif name == "gravityUp":
            self.gravityUpSound.play()
        elif name == "gravityDown":
            self.gravityDownSound.play()
        elif name == "gozerWins":
            self.gozerWinsSound.play()
        elif name == "frogWins":
            self.frogWinsSound.play()
        elif name == "eclipse":
            self.eclipseSound.play()

    def addBeginAndBack(self):
        """
            Add start and begin buttons when user needs them to restart level.
        """
        self.add(self.beginButton, 550, 200)
        self.add(self.backButton, 550, 275)

    def startLevel(self, param):
        """
            Start a level and game music.
            param = which level to start
        """
        self.menuSound.stop()
        self.levelSound.play(loops=-1)
        self.playLevel(param)

    def beginStage(self, param):
        """
            User clicked that she would like to start the game.
            Unpause game, start playing.
        """
        self.remove(self.beginButton)
        self.remove(self.backButton)
        self.level.pausedForBegin = False
        self.level.currentTime = 0

    def stageSelect(self, param):
        """
            Add everything to the game for stage select.
        """
        self.removeOldScreen()
        self.currentScreen = "SELECT"
        self.add(self.levelImage, 300, 0)
        self.add(self.selectLevelOne, 225, 225)
        self.add(self.selectLevelTwo, 625, 225)
        self.add(self.selectLevelThree, 825, 225)
        self.add(self.selectLevelFour, 625, 300)
        self.add(self.selectLevelFive, 425, 300)
        self.add(self.selectLevelSix, 25, 300)
        self.add(self.selectLevelSeven, 1025, 300)
        self.add(self.selectLevelEight, 425, 225)
        self.add(self.selectLevelNine, 825, 300)
        self.add(self.selectLevelTen, 225, 300)
        self.add(self.playerOneInput, 50, 25)
        self.add(self.playerOneTagline, 50, 100)
        self.add(self.playerTwoInput, 950, 25)
        self.add(self.playerTwoTagline, 950, 100)
        self.add(self.stageSelectToStartScreen, 550, 375)

    def creditsScreen(self, param):
        """
            Add everything to the game for credits screen.
        """
        self.removeOldScreen()
        self.currentScreen = "CREDITS"
        self.add(self.creditsTextArea, 300, 0)
        self.add(self.stageSelectToStartScreen, 550, 375)

    def backToStageSelect(self, param):
        """
            Return to stage select, but start music.
            This is used when user goes from game playing back to stage select.
        """
        self.removeOldScreen()
        self.levelSound.stop()
        self.menuSound.play(loops=-1)
        self.currentScreen = "SELECT"
        self.add(self.levelImage, 300, 0)
        self.add(self.selectLevelOne, 225, 225)
        self.add(self.selectLevelTwo, 625, 225)
        self.add(self.selectLevelThree, 825, 225)
        self.add(self.selectLevelFour, 625, 300)
        self.add(self.selectLevelFive, 425, 300)
        self.add(self.selectLevelSix, 25, 300)
        self.add(self.selectLevelSeven, 1025, 300)
        self.add(self.selectLevelEight, 425, 225)
        self.add(self.selectLevelNine, 825, 300)
        self.add(self.selectLevelTen, 225, 300)
        self.add(self.playerOneInput, 50, 25)
        self.add(self.playerOneTagline, 50, 100)
        self.add(self.playerTwoInput, 950, 25)
        self.add(self.playerTwoTagline, 950, 100)
        self.add(self.stageSelectToStartScreen, 550, 375)

    def removeOldScreen(self):
        """
            Remove old screen from GUI.
        """
        self.playSoundEffect("jump")
        if self.currentScreen == "NONE":
            pass
        elif self.currentScreen == "START":
            self.remove(self.startImage)
            self.remove(self.playButton)
            self.remove(self.instructionsButton)
            self.remove(self.creditsButton)
            self.remove(self.quitButton)
        elif self.currentScreen == "CREDITS":
            self.remove(self.creditsTextArea)
            self.remove(self.stageSelectToStartScreen)
        elif self.currentScreen == "SELECT":
            #self.remove(self.stageSelectLabel)
            self.remove(self.levelImage)
            self.remove(self.selectLevelOne)
            self.remove(self.selectLevelTwo)
            self.remove(self.selectLevelThree)
            self.remove(self.selectLevelFour)
            self.remove(self.selectLevelFive)
            self.remove(self.selectLevelSix)
            self.remove(self.selectLevelSeven)
            self.remove(self.selectLevelEight)
            self.remove(self.selectLevelNine)
            self.remove(self.selectLevelTen)
            self.remove(self.playerOneInput)
            self.remove(self.playerOneTagline)
            self.remove(self.playerTwoInput)
            self.remove(self.playerTwoTagline)
            self.remove(self.stageSelectToStartScreen)
        elif self.currentScreen == "GAME":
            self.remove(self.snakeButton)
            self.remove(self.darknessButton)
            self.remove(self.gravityDownButton)
            self.remove(self.gravityUpButton)
            self.remove(self.dietyPowerBar)
            self.remove(self.beginButton)
            self.remove(self.backButton)
            self.level = None
        elif self.currentScreen == "FROGINSTRUCT":
            self.remove(self.frog2Gozer)
            self.remove(self.frog2Title)
            self.remove(self.frogInstructionP)
        elif self.currentScreen == "GOZERINSTRUCT":
            self.remove(self.gozer2Frog)
            self.remove(self.frog2Title)
            self.remove(self.gozerInstructionP)

    def frogInstructions(self, param):
        """
            Add everything for the frog instructions screen.
        """
        self.removeOldScreen()
        self.currentScreen = "FROGINSTRUCT"
        self.add(self.frog2Gozer, 0, 350)
        self.add(self.frog2Title, 1050, 350)
        self.add(self.frogInstructionP, 0, 0)

    def gozerInstructions(self, param):
        """
            Add everything for the Osiris instruction screen.
        """
        self.removeOldScreen()
        self.currentScreen = "GOZERINSTRUCT"
        self.add(self.gozer2Frog, 0, 350)
        self.add(self.frog2Title, 1050, 350)
        self.add(self.gozerInstructionP, 0, 0)

    def startScreen(self, param):
        """
            Add everything for the start screen.
        """
        self.removeOldScreen()
        self.currentScreen = "START"
        self.add(self.startImage, 0, 0)
        self.add(self.playButton, 125, 300)
        self.add(self.instructionsButton, 375, 300)
        self.add(self.creditsButton, 625, 300)
        self.add(self.quitButton, 875, 300)

    def playLevel(self, levelName):
        """
            Add everything for the game screen.
            Set player names.
        """
        self.removeOldScreen()
        self.currentScreen = "GAME"

        self.add(self.beginButton, 550, 200)
        self.add(self.backButton, 550, 275)
        self.add(self.snakeButton, 1000, 0)
        self.add(self.darknessButton, 1000, 100)
        self.add(self.gravityUpButton, 1000, 200)
        self.add(self.gravityDownButton, 1000, 300)
        self.add(self.dietyPowerBar, 400, 25)
        self.level = PlatformerLevel(filename=levelName, width=1200, height=400, myMaster=self)
        self.level.frogName = self.playerOneInput.value
        self.level.gozerName = self.playerTwoInput.value

    def setPower(self, val):
        """
            Set mana bar to contain 'val' much mana.
        """
        self.dietyPowerBar.value = val

    def getPower(self):
        """
            Return how much mana is available for use.
        """
        return self.dietyPowerBar.value

    def tick(self, dt):
        """
            Update GUI logic.
        """
        if self.level != None:
            self.level.tick(dt)

    def handleEvent(self, e):
        """
            Relay user events to level.
        """
        if self.level != None:
            self.level.handleEvent(e)

    def drawGame(self, screen):
        """
            Draw game if possible.
        """
        if self.level != None:
            self.level.drawGame(screen)

    def snakeClicked(self, param):
        """
            User clicked summon snake button, inform game.
        """
        self.level.handleSnake()

    def darknessClicked(self, param):
        """
            User clicked eclipse button, inform game.
        """
        self.level.handleDarkness()

    def gravityUpClicked(self, param):
        """
            User clicked increase gravity button, inform game.
        """
        self.level.handleGravityUp()

    def gravityDownClicked(self, param):
        """
            User clicked decrease gravity button, inform game.
        """
        self.level.handleGravityDown()

    def exitGame(self, param):
        """
            Quit application.
        """
        sys.exit(0)

        

if __name__ == "__main__":
    #initialize python
    pygame.init()
    pygame.display.set_caption("Mr. Frog vs OSIRIS, LORD OF THE UNDERWORLD")
    #create game in full screen mode
    screen = pygame.display.set_mode((1200,400), SWSURFACE | FULLSCREEN)
    #screen = pygame.display.set_mode((1200, 400), SWSURFACE)
    app = gui.App()
    wc = FrogContainer()
    c = gui.Container(align=-1, valign=-1)
    c.add(wc, 0, 0)
    app.init(c)
    clock = pygame.time.Clock()
    done = False
    #game logic loop:
    #handle events
    #update time
    #draw
    while not done:
        for e in pygame.event.get():
            if e.type is QUIT:
                done = True
            else:
                wc.handleEvent(e)
                app.event(e)
        dt = clock.tick()
        wc.tick(dt)
        screen.fill((76, 120, 136))
        wc.drawGame(screen)
        app.paint(screen)
        pygame.display.flip()
        pygame.time.wait(10)
